#!/usr/bin/env python

import unittest
from math import sqrt, pi

from generic_boundary_conditions import *
from config import epsilon
from Numeric import allclose, array

        
class TestCase(unittest.TestCase):
    def setUp(self):
        pass
        #print "  Setting up"
        
    def tearDown(self):
        pass
        #print "  Tearing down"


    def test_generic(self):
        b = Boundary()

        try:
            b.evaluate()
        except:
            pass
        else:
            raise 'Should have raised exception'
        

    def test_dirichlet_empty(self):

        try:
            Bd = Dirichlet_boundary()
        except:
            pass
        else:
            raise 'Should have raised exception'

    def test_dirichlet(self):
        x = [3.14,0,0.1]
        Bd = Dirichlet_boundary(x)

        q = Bd.evaluate()
        assert allclose(q, x)


    def test_transmissive(self):
        from domain import Domain
        from quantity import Quantity
        
        a = [0.0, 0.0]
        b = [0.0, 2.0]
        c = [2.0,0.0]
        d = [0.0, 4.0]
        e = [2.0, 2.0]
        f = [4.0,0.0]

        points = [a, b, c, d, e, f]

        #bac, bce, ecf, dbe
        elements = [ [1,0,2], [1,2,4], [4,2,5], [3,1,4] ]
        
        domain = Domain(points, elements)
        domain.check_integrity()

        
        domain.quantities['level'] =\
                                   Quantity(domain, [[1,2,3], [5,5,5],
                                                     [0,0,9], [-6, 3, 3]])

        domain.quantities['ymomentum'] =\
                                   Quantity(domain, [[1,2,3], [5,5,5],
                                                     [0,0,9], [-6, 3, 3]])


        domain.check_integrity()

        #Test transmissve bdry
        try:
            T = Transmissive_boundary()
        except:
            pass
        else:
            raise 'Should have raised exception'

        T = Transmissive_boundary(domain)

        from config import default_boundary_tag
        domain.set_boundary( {default_boundary_tag: T} )

        assert len(domain.boundary) == len(domain.boundary_segments)
        assert len(domain.boundary_objects) == len(domain.boundary_segments)        

        
#-------------------------------------------------------------
if __name__ == "__main__":
    suite = unittest.makeSuite(TestCase,'test')
    runner = unittest.TextTestRunner()
    runner.run(suite)


        
